//! Mock OAuth provider.

use rocket::{form::Form, response::Redirect, serde::json::Json, FromForm, State};
use serde::Serialize;
use std::sync::atomic::{AtomicUsize, Ordering};

struct UserCounter(AtomicUsize);

pub fn mock_server(port: u16) -> tokio::runtime::Runtime {
    let config = rocket::config::Config {
        port,
        ..Default::default()
    };
    let server = rocket::custom(config)
        .manage(UserCounter(AtomicUsize::new(0)))
        .mount("/", rocket::routes![authorize, access_token, user]);

    let runtime = tokio::runtime::Runtime::new().unwrap();
    runtime.spawn(server.launch());
    runtime
}

#[rocket::get("/login/oauth/authorize?<client_id>&<redirect_uri>&<scope>&<response_type>&<state>")]
fn authorize(
    #[allow(unused)] client_id: String,
    redirect_uri: String,
    #[allow(unused)] scope: String,
    #[allow(unused)] response_type: String,
    state: String,
) -> Redirect {
    Redirect::to(format!("{}?code=foo&state={}", redirect_uri, state))
}

#[derive(FromForm)]
struct AccessTokenData {
    #[allow(unused)]
    grant_type: String,
    #[allow(unused)]
    code: String,
    #[allow(unused)]
    redirect_uri: String,
}

#[derive(Debug, Serialize)]
struct TokenInfo {
    access_token: String,
    // and more fields that we don't need.
}

#[rocket::post("/login/oauth/access_token", data = "<data>")]
fn access_token(#[allow(unused)] data: Form<AccessTokenData>) -> Json<TokenInfo> {
    let access_token = "mocktoken".to_string();
    Json(TokenInfo { access_token })
}

#[derive(Debug, Serialize)]
struct User {
    login: String,
}

#[rocket::get("/user")]
fn user(counter: &State<UserCounter>) -> Json<User> {
    let user_number = counter.0.fetch_add(1, Ordering::SeqCst);
    let login = format!("user-{}", user_number);
    Json(User { login })
}
